/*#############################
 *	Author: Mark Walker
 *	Class: BendySpring.java
 *		Class for springs that will attempt to return its object to a specific x,y coordinate
 *############################*/

import java.util.*;

public class BendySpring extends RotationalSpring implements Constraint
{
	protected double x0,y0,bX,bY;		//x0,y0 are the coordinates "aimed for; bX and bY are damping coefficients
	
	public BendySpring(PointMass o, Vector2D coords, double k1, double x, double y,double dampX, double dampY)
	{
		super(o,coords,k1,0,0);
		x0 = x;
		y0 = y;
		bX = dampX;
		bY = dampY;
	}
	
	public boolean updateConstraint(double dt, PhysicsEnv e)
	{
		//System.out.println("e = " + e + "; e.getLoc(obj) = " + e.getLoc(obj));
		Vector2D r = e.getLoc(obj).subtract(loc);
		
		double fx = k*(-r.getI()+x0) - bX*obj.getVelocity().getI();
		double fy = k*(-r.getJ()+y0) - bY*obj.getVelocity().getJ();
		Vector2D f = new Vector2D(fx,fy);
		
		obj.applyForce(f);
		
		//System.out.println("Applying spring force: " + new Vector2D(fx,fy) + "; k = " + k);
		
		return true;
	}
	
	public void addLength(double d)
	{
		Vector2D r = new Vector2D(x0,y0);
		double mag = r.magnitude();
		if (mag == 0 && d > 0)
			r = new Vector2D(0,d);
		else
			if (mag + d > 0)
			r.setMag(mag+d);
			else
			{
				r = new Vector2D(0,0);
			}
		System.out.println("x0,y0 = " + r);
		x0 = r.getI();
		y0 = r.getJ();
	}
	
	public double getK()
	{
		return k;
	}
	
	public void setK(double j)
	{
		k = j;
	}
}