/*#############################
 *	Author: Mark Walker
 *	Class: ConstraintsDemo1.java
 *		A simple constraints demonstration
 *############################*/
import java.util.*;
public class ConstraintsDemo2 implements GameEngine
{
	private PhysicsEngine2 phys;
	private BoundingCircle circle,circle2,circle3;
	
	private int HEIGHT = 70, WIDTH = 70, MASS = 1, MBOX = 100000, GRAVITY = 500, V_EXPL = 500, FORCE = 100, RADIUS = 1,
		ROWS = 7, COLS = 5, SPACE = 20, OFFSET = 200, X0 = 0, Y0 = 300; 
	private double K = .5, B = .1, BX = .03, BY = .03;
	public boolean bStatic, bBoxes;
	
	public ConstraintsDemo2()
	{
		phys = new PhysicsEngine2();
		
		circle = new BoundingCircle(MBOX,RADIUS,new Vector2D(500,500));
		getEnv().addObject(circle);
		/*circle2 = new BoundingCircle(MBOX,RADIUS,new Vector2D(600,300));
		getEnv().addObject(circle2);
		circle3 = new BoundingCircle(MBOX,RADIUS,new Vector2D(400,500));
		getEnv().addObject(circle3);
		DoubleConstraint c1 = new DoubleConstraint(circle,circle2,
								getEnv().getLoc(circle).subtract(getEnv().getLoc(circle2)).magnitude());
		getEnv().addObject(c1);
		DoubleConstraint c2 = new DoubleConstraint(circle3,circle,
								getEnv().getLoc(circle3).subtract(getEnv().getLoc(circle)).magnitude());
		getEnv().addObject(c2);
		DoubleConstraint c3 = new DoubleConstraint(circle3,circle2,
								getEnv().getLoc(circle3).subtract(getEnv().getLoc(circle2)).magnitude());
		getEnv().addObject(c3);*/
		
		circle.setAccel(new Vector2D(0,GRAVITY));
		//circle2.setAccel(new Vector2D(0,GRAVITY));
		//circle3.setAccel(new Vector2D(0,GRAVITY));
		
		AABB b = new AABB(1,900,200,new Vector2D(400,800));		//border boxes
		b.setStatic(true);
		AABB f = new AABB(1,900,200,new Vector2D(400,0));
		f.setStatic(true);
		AABB d = new AABB(1,200,900,new Vector2D(0,400));
		d.setStatic(true);
		AABB e = new AABB(1,200,900,new Vector2D(800,400));
		e.setStatic(true);
		getEnv().addObject(b);
		getEnv().addObject(f);
		getEnv().addObject(d);
		getEnv().addObject(e); 
		
		BoundingCircle c,cLast = null;
		DoubleConstraint dc;
		for (int i = 0; i < ROWS; i++)
		{
			for (int j = 0; j < COLS; j++)
			{
				c = new BoundingCircle(MASS,RADIUS,
					new Vector2D(j*SPACE+OFFSET,i*SPACE+OFFSET));
				getEnv().addObject(c);
				c.setAccel(new Vector2D(0,GRAVITY));
				//c.collisionOff();
				if (cLast != null)
				{
					dc = new DoubleConstraint(cLast,c,10);
					//getEnv().getLoc(cLast).subtract(getEnv().getLoc(c)).magnitude());
					
					getEnv().addObject(dc);
				} else
				{
					RigidRope rr = new RigidRope(c,new Vector2D(400,400),5);
					//getEnv().addObject(rr);
				}
				
				//dc = new DoubleConstraint(circle,c,
				//	getEnv().getLoc(circle).subtract(getEnv().getLoc(c)).magnitude());
				//getEnv().addObject(dc);
				cLast = c;
			}
		}
		dc = new DoubleConstraint(circle,cLast,10);
		//getEnv().getLoc(circle).subtract(getEnv().getLoc(cLast)).magnitude());
		getEnv().addObject(dc);
	}
	
	public void changeStaticMode()
	{
		System.out.println("Static mode changed.");
		bStatic = !bStatic;
	}
	
	public boolean bBoxes()
	{
		return bBoxes;
	}
	
	public void addObject(int x, int y)
	{
		if (bBoxes)
			addBox(x,y);
		else
			addCircle(x,y);
	}
	
	public void toggleBoxes()
	{
		//System.out.println("Object type toggled.");
		bBoxes = !bBoxes;
	}
	
	public void addBox(int x, int y)
	{
		//System.out.println("Adding box at location (" + x + ", " + y + ")");
		AABB box = new AABB(MASS,WIDTH,HEIGHT,new Vector2D(x,y));
		getEnv().addObject(box);
		box.setAccel(new Vector2D(0,GRAVITY));	
		if (bStatic)
			box.setStatic(true);	
	}
	
	public void addCircle(int x, int y)
	{
		//System.out.println("Adding circle at location (" + x + ", " + y + ")");
		BoundingCircle circle = new BoundingCircle(MASS,RADIUS,new Vector2D(x,y));
		getEnv().addObject(circle);
		circle.setAccel(new Vector2D(0,GRAVITY));	
		if (bStatic)
			circle.setStatic(true);	
	}
	
	public void explosion(int x, int y)
	{
		//System.out.println("EXPLOSION!");
		ArrayList l = getEnv().getAllObjects();
		AABB box;
		BoundingCircle circle;
		for (int i = 0; i < l.size(); i++)
		{
			if (((PointMass)l.get(i)).CLASS() == 2)
			{
				box = (AABB)l.get(i);
				if (!box.bStatic() || box.bSleeping())
				{
					box.setVelocity(new Vector2D(V_EXPL*-Math.sin(getEnv().getLoc(box).getI()/2),
						V_EXPL*-Math.cos(getEnv().getLoc(box).getJ()/2)));
					box.wake();
				}
			} else if (((PointMass)l.get(i)).CLASS() == 1)
			{
				circle = (BoundingCircle)l.get(i);
				if (!circle.bStatic() || circle.bSleeping())
				{
					circle.setVelocity(new Vector2D(V_EXPL*-Math.sin(getEnv().getLoc(circle).getI()/2),
					//-V_EXPL));
					V_EXPL*-Math.cos(getEnv().getLoc(circle).getJ()/2)));
					circle.wake();
				}
			}
		}
	}
	
	public void hitBox(Vector2D d)
	{
		circle.applyForce(d.scalarMult(FORCE));
	}
	
	public void act()
	{
		phys.act();
		if (circle.bSleeping())
			circle.wake();
		ArrayList l = getEnv().getAllObjects();
		double y = getEnv().getLoc(circle).getJ();
		PointMass p;
		for (int i = 0; i < l.size() && circle.getFlag(); i++)
		{
			p = (PointMass)l.get(i);
			if (y > getEnv().getLoc(p).getJ())
				p.wake();
		}
	}
	
	public PhysicsEnv getEnv()
	{
		return phys.getEnv();
	}
}