import java.util.*;

public class DoubleConstraint
{
	protected PointMass objA, objB;
	private double length;
	
	public DoubleConstraint(PointMass a, PointMass bN, double l)
	{
		objA = a;
		objB = bN;
		length = l;
	}
	
	public boolean act(PhysicsEnv e)
	{
		if (objA.bSleeping() != objB.bSleeping())
		{
			objA.wake();
			objB.wake();
		}
			
		Vector2D locA = e.nextLoc(objA,.01), locB = e.nextLoc(objB,.01);
		if (locA.subtract(locB).squaredMag() < length*length)
			return false;
		double E = 0;
		PointMass a = objA, b = objB;
		double vya = a.getVelocity().getJ(), vxa = a.getVelocity().getI();
		a.getVelocity().setJ( (((a.getVelocity().getJ()*(a.getMass()-(E*b.getMass())))/(a.getMass()+b.getMass())) +
			((b.getVelocity().getJ()*(b.getMass()*(1+E)))/(a.getMass()+b.getMass()))));
				
		b.getVelocity().setJ( (((b.getVelocity().getJ()*(b.getMass()-(E*a.getMass())))/(a.getMass()+b.getMass())) +
			((vya*(a.getMass()*(1+E)))/(a.getMass()+b.getMass()))));
		
		a.getVelocity().setI( (((a.getVelocity().getI()*(a.getMass()-(E*b.getMass())))/(a.getMass()+b.getMass())) +
			((b.getVelocity().getI()*(b.getMass()*(1+E)))/(a.getMass()+b.getMass()))));
		
		b.getVelocity().setI( (((b.getVelocity().getI()*(b.getMass()-(E*a.getMass())))/(a.getMass()+b.getMass())) +
			((vxa*(a.getMass()*(1+E)))/(a.getMass()+b.getMass()))));
			
		//fix positions
		Vector2D c = locA.subtract(locB).scalarDiv(2);
		double dL = c.magnitude()-(length/2);
		c = c.scalarDiv(c.magnitude()).scalarMult(dL);
		e.moveObject(objB,c);
		e.moveObject(objA,c.scalarMult(-1));
		
		return true;
	}
	
	public boolean isNull()
	{
		return (objA == null) || (objB == null);
	}
	
	public PointMass getObjA()
	{
		return objA;
	}
	
	public PointMass getObjB()
	{
		return objB;
	}
	
	public double getK()
	{
		return 0;
	}
	
	public void setK(double j)
	{
	}
}