import java.util.*;

public class GravityGame implements GameEngine
{
	private PhysicsEngine2 phys;
	private ArrayList objs;
	private BoundingCircle player;
	
	//############## MAIN PARAMETERS ################
	private final int MASS = 1000000, MIN_SCORE = -100,MAX_MASS = 1000000000, MAX_WAVE = 25, BIG_MASS = 100000000, RADIUS = 400, VAR = 300, WAVE_INCR = 5, MAX_DIM = 800,DIM=400;
	private final double SPEED = 200, ACCEL = 150, PLAYER_RAD = 20,PLANET_RAD = 7,MASS_INC = 1.03;
	private int wave, time, score,highScore;
	
	public GravityGame()
	{
		wave = 3;
		time = 20;
		score = 0;
		highScore = 0;
		phys = new PhysicsEngine2();
		objs = new ArrayList();
		
		AABB b = new AABB(1,800,100,new Vector2D(400,800));		//border boxes
		b.setStatic(true);
		AABB c = new AABB(1,800,100,new Vector2D(400,0));
		c.setStatic(true);
		AABB d = new AABB(1,100,800,new Vector2D(0,400));
		d.setStatic(true);
		AABB e = new AABB(1,100,800,new Vector2D(800,400));
		e.setStatic(true);
		getEnv().addObject(b);
		getEnv().addObject(c);
		getEnv().addObject(d);
		getEnv().addObject(e);
		
		player = new BoundingCircle(BIG_MASS,PLAYER_RAD,new Vector2D(400,400));
		getEnv().addObject(player);
		//player.collisionOff();
		
		System.out.println("GravityGame:: MAX_MASS = " + MAX_MASS + "; MIN_SCORE = " + MIN_SCORE);	
	}
	
	public int getScore()
	{
		return score;
	}
	
	public int getNum()
	{
		return objs.size();
	}
	
	public void spawnWave()
	{
		System.out.println("Spawning wave...");
		double t = 0, r;
		for (int i = 0; i < wave; i++)
		{
			r = (Math.random()*VAR-(VAR/2))+RADIUS;
			t += 6.28318/(wave);
			objs.add( new BoundingCircle(MASS,PLANET_RAD,new Vector2D(Math.cos(t)*r + 400, Math.sin(t)*r + 400)) );
			getEnv().addObject((BoundingCircle)objs.get(i));
			//((BoundingCircle)objs.get(i)).collisionOff();		//comment out to try collision detection
			((BoundingCircle)objs.get(i)).setVelocity(new Vector2D(-Math.sin(t)*SPEED,Math.cos(t)*SPEED));
			((BoundingCircle)objs.get(i)).collisionOff();
		}
		wave+= WAVE_INCR;
		if (wave > MAX_WAVE)
			wave = MAX_WAVE;
	}
	
	public ArrayList getObjs()
	{
		return objs;
	}
	
	public int getHighScore()
	{
		return highScore;
	}
	
	public BoundingCircle getPlayer()
	{
		return player;
	}
	
	public void movePlayer(Vector2D dir)
	{
		player.applyForce(dir.scalarMult(ACCEL));
		//System.out.println("Moving player: " + dir);
	}
	
	public int cleanUp()
	{
		ArrayList l = getEnv().getAllObjects();
		for (int i = 0; i < objs.size(); i++)
		{
			if (!l.contains(objs.get(i)))
				objs.remove(i);
		}
		int n = 0;
		PointMass c;
		Vector2D loc;
		for (int i = 0; i < objs.size(); i++)
		{
			c = (PointMass)objs.get(i);
			if (c == null)
			{
				//objs.remove(i);
			} else
			{
				loc = getEnv().getLoc(c);
				if (loc.getI() > MAX_DIM + DIM || loc.getI() < DIM-MAX_DIM ||
					loc.getJ() > MAX_DIM + DIM || loc.getJ() < DIM-MAX_DIM)
				{
					System.out.println("Removing object out of bounds at: " + loc);
					getEnv().removeObject(c);
					objs.remove(i);
					score+=10;
				} else
					n++;
			}
		}
		//System.out.println("Removed " + n + " objects");
		return objs.size();
	}
	
	public boolean lost()
	{
		return (score <= MIN_SCORE) || (player.getMass() >= MAX_MASS);
	}
	
	public void act()
	{
		if (player.bSleeping())
			player.wake();
		/*Vector2D pLoc = getEnv().getLoc(player);
		if (pLoc.getI() < 0)
		
			pLoc.setI(0);
		if (pLoc.getJ() < 0)
			pLoc.setJ(0);
		if (pLoc.getI() > DIM*2)
			pLoc.setI(DIM*2);
		if (pLoc.getJ() > DIM*2)
			pLoc.setJ(DIM*2);*/
		
		if (cleanUp() == 0)
			spawnWave();
		
		double r;
		Vector2D p = new Vector2D(0,0), ray;
		BoundingCircle a, b;
		player.setAccel(0,0);
		for (int i = 0; i < objs.size(); i++)
		{
			a = (BoundingCircle)objs.get(i);
			if (a != null)
			{
				a.setAccel(0,0);		//reset acceleration vector every time to 0
				for (int j = 0; j < objs.size(); j++)
				{
					b = (BoundingCircle)objs.get(j);
					if (j != i && b != null)		//add accelerations in the direction of each other object, (M/R^2)
					{
						p = getEnv().getLoc(b).subtract(getEnv().getLoc(a));
						r = p.magnitude();
						if (r < 1)
						{
							r = 1;
						}
						p.setMag(b.getMass()/(r*r));
						a.accelerate(p);
					}
				}
				
				p = getEnv().getLoc(player).subtract(getEnv().getLoc(a));
				r = p.magnitude();
				ray = PhysicsEngine2.rayTrace(a,getEnv().getLoc(a),a.getVelocity().scalarMult(0.01),getEnv());
				
				//if (ray != null)
					//System.out.println("Ray = " + ray);
				
				if (r < player.getRadius() + a.getRadius() || (ray != null &&
					ray.magnitude() <= a.getVelocity().scalarMult(0.01).magnitude()
					&& getEnv().getLoc(a).add(ray).subtract(getEnv().getLoc(player)).magnitude() <= PLAYER_RAD+1))
				{
					player.setMass(player.getMass()*MASS_INC);
					getEnv().removeObject(a);
					objs.remove(i);
					score -= 20;
					System.out.println("Point mass got sucked in.");
				} else
				{
					if (r < 1)
						r = 1;
					p = p.scalarDiv(p.magnitude()).scalarMult(player.getMass()/(r*r));
					a.accelerate(p);
					
					p = p.scalarMult(-1);
					p = p.scalarDiv(p.magnitude()).scalarMult(a.getMass()/(r*r));
					player.accelerate(p);
				}
			}
		}
		phys.act();
		
		if (score > highScore)
			highScore = score;
	}
	
	public PhysicsEnv getEnv()
	{
		return phys.getEnv();
	}
}