/*#############################
 *	Author: Mark Walker
 *	Class: RightTriangle.java
 *		Right triangle class
 *############################*/
public class RightTriangle extends PointMass			//Axis-Aligned Bounding Box class
{
	private Vector2D h, b, n;	//base and height
	private int CLASS = 4;
	
	public RightTriangle(double mass, int width, int height, Vector2D l)	//l is location vector for the CENTER
	{																//of the triangle
		super(mass,l);
		b = new Vector2D(width,0);
		h = new Vector2D(0,-height);
		n = new Vector2D(-height,width);
		n = n.scalarDiv(n.magnitude());
		System.out.println("New RightTriangle:: b = " + b + "; h = " + h + "; n = " + n);
		setStatic(true);
	}
	
	public boolean collideCircle(BoundingCircle c, Vector2D loc, double E, PhysicsEnv e)	//loc is b's coords local to this triangle
	{
		//System.out.println("RightTriangle:: testing collideCircle at " + loc);
		if (!quickCircleTest(c.getRadius(),loc.getI(),loc.getJ()))
			return false;
		//System.out.println("RightTriangle:: quick test passed...");
		double d = PhysicsEngine2.pointLineSqDist(h,new Vector2D(0,0),loc);		//test vertical side
		
		if (d <= c.getRadius()*c.getRadius())
		{
			//System.out.println("d1 = " + d);
			//System.out.println("	old v = " + c.getVelocity());
			/*if (!PhysicsEngine2.AASideOverlap(loc.getJ(),loc.getJ(),getHeightVector().getJ(),0))
				c.getVelocity().setJ(-c.getVelocity().getJ() * E);
			else
				c.getVelocity().setI(-c.getVelocity().getI() * E);*/
			if (getBase()*loc.getI() <= 0 && loc.getJ()-getHeight() < 0)
			{
				Vector2D nV = loc.subtract(h);
				nV = nV.scalarDiv(nV.magnitude()).scalarMult(c.getVelocity().magnitude()*E);
				c.getVelocity().setJ(nV.getJ());
				c.getVelocity().setI(nV.getI());
				//System.out.println("	new v = " + c.getVelocity() +"loc-height = " + loc.getJ() + " - " + getHeight());
				return true;
			} else if (getBase()*loc.getI() <=0 && 
				PhysicsEngine2.AASideOverlap(loc.getJ(),loc.getJ(),0,getHeight()))
			{
				c.getVelocity().setI(-c.getVelocity().getI()*E);
				return true;
			}
		}
		
		d = PhysicsEngine2.pointLineSqDist(new Vector2D(0,0),b,loc);		//test bottom
		
		if (d <= c.getRadius()*c.getRadius())
		{
			//System.out.println("d2 = " + d);
			//System.out.println("	old v = " + c.getVelocity());
			if (PhysicsEngine2.AASideOverlap(loc.getI(),loc.getI(),0,getBase()))
				c.getVelocity().setJ(-c.getVelocity().getJ()*E);
			else
				c.getVelocity().setI(c.getVelocity().getI()*E);
			//System.out.println("	new v = " + c.getVelocity());
			return true;
		}
		
		d = PhysicsEngine2.pointLineSqDist(b,h,loc);						//test hypotenuse
		if (d <= c.getRadius()*c.getRadius())
		{
			//System.out.println("d3 = " + d);
			//System.out.println("	old v = " + c.getVelocity());
			Vector2D v = c.getVelocity();
			Vector2D vf = v.subtract(n.scalarMult(v.dot(n)));
			c.getVelocity().set(vf.getI(),vf.getJ());
			//System.out.println("	new v = " + c.getVelocity());
			e.moveObject(c,n.scalarMult(-1));
			return true;
		}
		//System.out.println("	Collision turned out to be false.");
		return false;
	}
	
	private boolean quickCircleTest(double r, double x, double y)	//quick 'n easy square distance test
	{																		//NOT a final test
		//find squared distance from closest point to the circle center
		//and see if it is less than the squared radius.
		double d;
		if (getHeight() > getBase())
			d = getHeight();
		else
			d = getBase();
		
		if (Math.sqrt(getHeight()*getHeight() + getBase()*getBase()) > Math.sqrt(x*x + y*y) - r)
			return true;
		return false;
	}
	
	public boolean collideOBB(OBB b, double x, double y)
	{
		return false;
	}
	public boolean collideAABB(AABB b, Vector2D loc,double E,PhysicsEnv e)
	{
		BoundingCircle c = new BoundingCircle(1,b.getHeightVector().add(b.getWidthVector()).magnitude(),loc);
		if (!quickCircleTest(c.getRadius(),loc.getI(),loc.getJ()))
			return false;
		/*Vector2D topLeft = loc.subtract(b.getWidthVector()).subtract(b.getHeightVector()), 
				topRight = loc.subtract(b.getHeightVector()).add(b.getWidthVector()),
				bottomLeft = loc.add(b.getHeightVector()).subtract(b.getWidthVector()),
				bottomRight = loc.add(b.getHeightVector()).add(b.getWidthVector());
		Vector2D locA = new Vector2D(loc.getI()+(getBase()/2),loc.getJ()+(getHeight()/2));
		AABB a = new AABB(1,getBase(),getHeight(),locA);
		
		Vector2D dir = PhysicsEngine2.AAFromSide(b,a);
		
		if (dir.getI()*getBase() > 0 || dir.getJ()*getHeight() > 0)
		{
			return false;
		}
		
		if (dir.getJ() != 0)
		{
			b.getVelocity().setJ(-b.getVelocity().getJ()*E);
			return true;
		}
		if (dir.getI() != 0)
		{
			b.getVelocity().setI(-b.getVelocity().getI()*E);
			return true;
		}
		System.out.println("Triangle/AABB collision error.");*/
		
		double d = PhysicsEngine2.pointLineSqDist(getBaseVector(),h,loc);			//test hypotenuse
		if (d <= c.getRadius()*c.getRadius())
		{
			//System.out.println("d3 = " + d);
			//System.out.println("	old v = " + c.getVelocity());
			Vector2D v = b.getVelocity();
			Vector2D vf = v.subtract(n.scalarMult(v.dot(n)*E));
			b.getVelocity().set(vf.getI(),vf.getJ());
			//System.out.println("	new v = " + c.getVelocity());
			e.moveObject(b,n.scalarMult(-1));
			return true;
		}
		
		return false;
	}
	
	public Vector2D getNormal() 	//returns triangle's hypotenuse normal vector
	{
		return n;
	}
	
	public int getBase()		//returns halfwidth variable
	{
		return (int)b.getI();
	}
	
	public int getHeight()		//returns halfheight variable
	{
		return (int)h.getJ();
	}
	
	public Vector2D getBaseVector()
	{
		return b;
	}
	
	public Vector2D getHeightVector()
	{
		return h;
	}
	
	public int CLASS()
	{
		return 4;
	}
}