




//OBSOLETE

/*#############################
 *	Author: Mark Walker
 *	Class: PhysicsEngine.java
 *		This is the old physics engine, no longer being used but saved just in case.
 *############################*/
import java.util.*;

public class PhysicsEngine
{
	private PhysicsEnv env;
	private long t;
	private double dt;
	//BACKSTEPS is maximum number of timesteps an object will be traced back in a collision
	
	//E is the "coefficient of restitution" (Palmer, 145), a number between 0 and 1 that controls
	//	dampening of collisions, where 1 means elastic, and 0 is completely inelastic (stuck together)
	
	//COLL_CORRECTION pushes an object back by this amount if it was originally going to be moved 
	//	by less than its value
	
	//MIN_SQVEL is the minimum squared velocity magnitude of an object before it will come to rest
	
	//REST_THRESH is the minimum value that both velocity components can be at the same time 
	//	before coming to rest
	private final double BACKSTEPS = 15, E = .8, STEP_SIZE = 1,
		COLL_CORRECTION = 0, MIN_SQVEL = 0, REST_THRESH = 0;
	
	public PhysicsEngine()
	{
		env = new PhysicsEnv();
		t = System.currentTimeMillis();
		dt = 0;
	}
	
	public void act()
	{
		//find delta time
		/*long t2 = System.currentTimeMillis();
		dt = (double)(t2 - t)/1000.0;
		if (dt == 0.0)			//don't run all this if no time has gone by
			return;
		t = t2;*/
		dt = .01;				//Comment out this line to do 'real' time simulation
		PointMass p, b;			//all p's being tested against b's
		boolean v;				//true if there was a collision between a and b
		Vector2D w;				//change in position for p
		for (int i = 0; i < env.getAllObjects().size(); i++)
		{
			p = (PointMass)(env.getAllObjects().get(i));
			if (p == null)
				return;
			w = p.act(dt);		//act and return change in position
			if (w.squaredMag() < MIN_SQVEL)
			{
				w.setI(0);
				w.setJ(0);
			}
			env.moveObject(p,w);		//update environment
			for (int j = 0; j < env.getAllObjects().size() && p.bCollidable(); j++)
			{
				if (j != i)			//do not collide with self
				{
					b = (PointMass)(env.getAllObjects().get(j));		//object being tested against
					if (b != null && !b.getFlag() && b.bCollidable())		//do not test objects flagged as tested
					{
						//System.out.println("Possible collision...");
						v = collision(p,b);			//find exact collision
						if (v && !p.bStatic())
						{
							//System.out.println("Collision!");		//collision response code goes here?System.out.println("6::a.getVelocity = " + p.getVelocity());
							if (env.getLoc(p).getI() - p.oldLoc().getI() < REST_THRESH 
								&& env.getLoc(p).getJ() - p.oldLoc().getJ() < REST_THRESH )
							{
								p.getVelocity().setI(0);
								p.getVelocity().setJ(0);
							}
						}
					}
				}
			}
			//p.flag();			//do not flag objects for now...
		}
	}
	
	private boolean collision(PointMass a, PointMass b)
	{
		//each type of object given a defined CLASS value (see PointMass header)
		//System.out.println("PhysicsEngine::60: identifying collision type...");
		if (a.bStatic() && b.bStatic())
			return false;
		if (a.CLASS() == 1)
		{
			if (b.CLASS() == 1)
				return collisionCC((BoundingCircle)a,(BoundingCircle)b);
			if (b.CLASS() == 2)
				return collisionCA((BoundingCircle)a,(AABB)b);
			if (b.CLASS() == 3)
				return collisionCO(a,b);
		}
		if (a.CLASS() == 2)
		{
			if (b.CLASS() == 1)
			{
				return collisionCA((BoundingCircle)b,(AABB)a);
			}
			if (b.CLASS() == 2)
				return collisionAA((AABB)a,(AABB)b);
			if (b.CLASS() == 3)
				return collisionAO(a,b);
		}
		if (a.CLASS() == 3)
		{
			if (b.CLASS() == 1)
			{
				return collisionCO(b,a);
			}
			if (b.CLASS() == 2)
			{
				return collisionAO(b,a);
			}
			if (b.CLASS() == 3)
				return collisionOO(a,b);
		}
		System.out.println("PhysicsEngine::108: failed to detect collision type.");
		return false;
	}
	
	public boolean collisionCC(BoundingCircle a, BoundingCircle b)
	{
		Vector2D av = a.getVelocity();
		Vector2D bv = b.getVelocity();
		if (a.collideCircle(b,(b.getLoc().getI() - a.getLoc().getI()), (b.getLoc().getJ() - a.getLoc().getJ())))
		{
			//vector c points to circle b (a at origin)
			Vector2D c = new Vector2D(b.getLoc().getI() - a.getLoc().getI(), b.getLoc().getJ() - a.getLoc().getJ());
			//System.out.println("C X AV = " + c.cross(av));
			
			//temporary collision response
			if (c.cross(av) > 0)		//is cross product of c and av positive?
			{
				a.setVelocity(av.rotateCCW(3.14 - 2*av.angle(c)));	//rotate velocity vector
			} else
			{
				a.setVelocity(av.rotateCCW(-3.14 + 2*av.angle(c)));	//else rotate velocity vector other way
			}
			if (c.cross(bv) > 0)		//is cross product of c and av positive?
			{
				a.setVelocity(bv.rotateCCW(3.14 - 2*bv.angle(c)));	//rotate velocity vector
			} else
			{
				b.setVelocity(bv.rotateCCW(-3.14 + 2*bv.angle(c)));	//else rotate velocity vector other way
			}
			
			//perfectly elastic collision, momentum conserved, energy conserved (Palmer, 145)
			b.getVelocity().setMag( ((a.getMass()-(E*b.getMass()))/(a.getMass()+b.getMass()))*b.getVelocity().magnitude() +
									((a.getMass()*(1+E))/(a.getMass()+b.getMass()))*a.getVelocity().magnitude() );
									
			a.getVelocity().setMag( ((b.getMass()-(E*a.getMass()))/(a.getMass()+b.getMass()))*a.getVelocity().magnitude() +
									((b.getMass()*(1+E))/(a.getMass()+b.getMass()))*b.getVelocity().magnitude() );
			
			//System.out.println("PhysicsEngine::139: CC collision TRUE.");
			return true;
		}
		//System.out.println("PhysicsEngine::117: CC collision FALSE.");
		return false;
	}
	
	//Collision between Circle and AABB
	public boolean collisionCA(BoundingCircle a, AABB b)
	{
		//bound AABB into a circle and do circle/circle test FIRST (faster)
		if (!a.collideCircle(new BoundingCircle(b.getMass(),
			Math.sqrt(b.getWidth()*b.getWidth()+b.getHeight()*b.getHeight()),
			env.getLoc(b)),env.getLoc(b).getI() - env.getLoc(a).getI(),
			env.getLoc(b).getJ() - env.getLoc(a).getJ()))
			return false;
		
		//more rigorous collision test
		boolean bCollide = false;
		
		Vector2D topLeft = env.getLoc(b).subtract(b.getWidthVector()).subtract(b.getHeightVector());
		Vector2D topRight = env.getLoc(b).add(b.getWidthVector()).subtract(b.getHeightVector());
		Vector2D bottomLeft = env.getLoc(b).subtract(b.getWidthVector()).add(b.getHeightVector());
		Vector2D bottomRight = env.getLoc(b).add(b.getWidthVector()).add(b.getHeightVector());
		
		if (pointLineSqDist(topLeft,topRight,env.getLoc(a)) < a.getRadius()*a.getRadius())
			bCollide = true;
		else if (pointLineSqDist(topLeft,bottomLeft,env.getLoc(a)) < a.getRadius()*a.getRadius())
			bCollide = true;
		else if (pointLineSqDist(topRight,bottomRight,env.getLoc(a)) < a.getRadius()*a.getRadius())
			bCollide = true;
		if (pointLineSqDist(bottomLeft,bottomRight,env.getLoc(a)) < a.getRadius()*a.getRadius())
			bCollide = true;
		
		if (!bCollide)
			return false;
		
		//System.out.println("Circle/AABB collision.");
		double xf, yf, x0, y0, bw, bh;
		if (!a.bStatic() || (a.getVelocity().getI() != 0 && a.getVelocity().getJ() != 0))
		{	
			//if a is not stationary
			xf = env.getLoc(a).getI() - env.getLoc(b).getI();	//old and new positions
			yf = env.getLoc(a).getJ() - env.getLoc(b).getJ();
			x0 = xf - (a.getVelocity().getI()+(a.getAcceleration().getI()*dt))*dt;
			y0 = yf - (a.getVelocity().getJ()+(a.getAcceleration().getJ()*dt))*dt;
			bw = b.getWidth();
			bh = b.getHeight();
		}
		else	//if a is stationary, move a to b with negative Vb
		{
			xf = env.getLoc(a).getI() - env.getLoc(b).getI();	//old and new positions
			yf = env.getLoc(a).getJ() - env.getLoc(b).getJ();
			x0 = xf + (b.getVelocity().getI()+(b.getAcceleration().getI()*dt))*dt;
			y0 = yf + (b.getVelocity().getJ()+(b.getAcceleration().getJ()*dt))*dt;
			bw = b.getWidth();
			bh = b.getHeight();
		}
		//System.out.println("xf = " + xf + "; yf = " + yf + "; x0 = " + x0 + "; y0 = " + y0);
		Vector2D v = CAFromSide(xf,yf,x0,y0,bw,bh);
		
		if (v.getI() != 0 && v.getJ() == 0)		//side collision
		{
			int s;		//sign of a with respect to b (x location)
			if (env.getLoc(a).getI() < env.getLoc(b).getI())
				s = -1;
			else
				s = 1;
			//how much a will be moved by (or b moved back by)
			double dx = env.getLoc(b).getI()+(b.getWidth()*s)-env.getLoc(a).getI()+(a.getRadius()*s);
			//System.out.println("Side collision (circle/AABB)...");
			if (!a.bStatic() && !b.bStatic())
			{
				double dOmega = ((.6666*a.getVelocity().getJ())/a.getRadius()),
				vafy = .6666*a.getVelocity().getJ(),
				vbfy = ((b.getI()*dOmega)/(b.getMass()*a.getRadius())) + b.getVelocity().getJ();
				
				a.getVelocity().setJ(vafy);
				b.getVelocity().setJ(vbfy);
				a.addOmega(dOmega);
				a.getVelocity().setI(-a.getVelocity().getI()*E);
				b.getVelocity().setI(-b.getVelocity().getI()*E);
				env.moveObject(a,new Vector2D(dx,0));
				env.moveObject(b,new Vector2D(-dx,0));
				return true;
			}
			if (a.bStatic())
			{
				env.moveObject(b,new Vector2D(-dx,0));			
				b.getVelocity().setI(-b.getVelocity().getI()*E);
				return true;
			}
			env.moveObject(a,new Vector2D(dx,0));
			double dOmega = ((.6666*a.getVelocity().getJ())/a.getRadius()),
				vafy = .6666*a.getVelocity().getJ(); 
			a.getVelocity().setJ(vafy);
			a.addOmega(dOmega);
			a.getVelocity().setI(-a.getVelocity().getI()*E);
			return true;
		}
		
		if (v.getJ() != 0 && v.getI() == 0)		//top/bottom collision
		{
			//System.out.println("Top/bottom collision (circle/AABB)...");
			int s;		//sign of a with respect to b (x location)
			if (env.getLoc(a).getJ() < env.getLoc(b).getJ())
				s = -1;
			else
				s = 1;
			double dOmega = ((.6666*a.getVelocity().getI())/a.getRadius());
			a.addOmega(dOmega);
			//how much a will be moved by (or b moved back by)
			double dy = env.getLoc(b).getJ()+(b.getHeight()*s)-env.getLoc(a).getJ()+(a.getRadius()*s);
			if (!a.bStatic() && !b.bStatic())
			{
					double vafx = .6666*a.getVelocity().getI(),
					vbfx = ((b.getI()*dOmega)/(b.getMass()*a.getRadius())) + b.getVelocity().getI();
				
				env.moveObject(a,new Vector2D(0,dy));
				env.moveObject(b,new Vector2D(0,-dy));
				a.getVelocity().setI(vafx);
				b.getVelocity().setI(vbfx);
				a.getVelocity().setJ(-a.getVelocity().getJ()*E);
				b.getVelocity().setJ(-b.getVelocity().getJ()*E);
				return true;
			}
			if (a.bStatic())
			{
				b.getVelocity().setJ(-b.getVelocity().getJ()*E);
				env.moveObject(b,new Vector2D(0,-dy));
				return true;
			}
			double vafx = .6666*a.getVelocity().getI(); 
			a.getVelocity().setI(vafx);
			a.addOmega(dOmega);
			a.getVelocity().setJ(-a.getVelocity().getJ()*E);
			env.moveObject(a,new Vector2D(a.getVelocity().getI()*dt,dy));
			return true;
		}
		//System.out.println("Corner collision.");
				
		Vector2D L;
		System.out.println("a loc = " + env.getLoc(a) + "; b loc = " + env.getLoc(b));
		if (env.getLoc(b).getJ() > env.getLoc(a).getJ())
			L = env.getLoc(b).subtract(b.getHeightVector());
		else
			L = env.getLoc(b).add(b.getHeightVector());
		System.out.println("L1 = " + L);
		if (env.getLoc(b).getI() > env.getLoc(a).getI())
			L = L.subtract(b.getWidthVector());
		else
			L = L.add(b.getWidthVector());
		System.out.println("L2 = " + L);
		L = L.subtract(env.getLoc(a));		//box b in local coordinates of circle a
		System.out.println("L = " + L);
		Vector2D dL = L.scalarDiv(L.magnitude()).scalarMult(a.getRadiusVector().magnitude()).subtract(L);
		System.out.println("dL = " + dL);
		env.moveObject(a,dL.scalarMult(-1));
		env.moveObject(b,dL.scalarMult(1));
		
		a.getVelocity().setJ(-a.getVelocity().getJ());
		a.getVelocity().setI(-a.getVelocity().getI());
		b.getVelocity().setJ(-b.getVelocity().getJ());
		b.getVelocity().setI(-b.getVelocity().getI());
		
		return true;	
	}
	
	private Vector2D CAFromSide(double xf, double yf, double x0, double y0, double bw, double bh)
	{	
		if (AASideOverlap(x0,x0,-bw,bw))	//if old center overlaps on x axis, return top/bottom
			return new Vector2D(0,y0);
		if (AASideOverlap(y0,y0,-bh,bh))	//if overlap on y axis, return side collision
			return new Vector2D(x0,0);
		//else came from a corner...
		if (AASideOverlap(xf,xf,-bw,bw))	//crossed to overlap on x
			return new Vector2D(0,yf);
		if (AASideOverlap(yf,yf,-bh,bh))	///...or on y
			return new Vector2D(xf,0);
		return new Vector2D(xf,yf);			//else intersected with corner first
	}
	
	public boolean collisionCO(PointMass a, PointMass b)
	{
		return false;
	}
	
	//Collision between two AABB's
	public boolean collisionAA(AABB a, AABB b)
	{
		//if 'a' is "static" or stationary, return false
		if (a.bStatic() || (a.getVelocity().getI() == 0 && a.getVelocity().getJ() == 0))
			return false;
		
		//check for collision
		if (!a.collideAABB(b,env.getLoc(b).getI()-env.getLoc(a).getI(),env.getLoc(a).getJ()-env.getLoc(b).getJ()))
			return false;
		//respond to collision
		Vector2D v = AAFromSide(a,b);		//which side did 'a' come from?
		
		if (v == null)
			return false;
		
		//if from side
		if (v.getI() != 0 && v.getJ() == 0)
		{
			//System.out.println("Side collision...");
			
			//move objects back as far as they need to be
			//System.out.println("	LocA = " +env.getLoc(a)+"; LocB = " +env.getLoc(b));
			
			v = env.getLoc(a).subtract(env.getLoc(b));
			if (v.getI() > 0)
			{
				v = v.subtract(a.getWidthVector());
				v = v.subtract(b.getWidthVector());
			}
			else
			{
				v = v.add(a.getWidthVector());
				v = v.add(b.getWidthVector());
			}
			System.out.println("		v0 = " + v);
			v.setI((double)((int)v.getI()));
			v.setJ(0);
			System.out.println("		v1 = " + v);
			
			if (v.getI() == 0)
				if (env.getLoc(a).getI() - env.getLoc(b).getI() > 0)
					v.setI(-1);
				else
					v.setI(1);
			
			/*if (v.getI() < COLL_CORRECTION && v.getI() > 0)
				v.setI(COLL_CORRECTION);
			else if (v.getI() > -COLL_CORRECTION && v.getI() < 0)
				v.setI(-COLL_CORRECTION);
			else if (v.getI() == 0)
				if (env.getLoc(a).getI() < env.getLoc(b).getI())
					v.setI(-COLL_CORRECTION);
				else
					v.setI(COLL_CORRECTION);*/
			
			env.moveObject(a,v.scalarMult(-1));
			if (!b.bStatic())
				env.moveObject(b,v.scalarMult(1));
				
			int s;
			/*if (env.getLoc(a).getI() - env.getLoc(b).getI() < 0)
				s = -1;
			else
				s = 1;
			v = new Vector2D(s*STEP_SIZE,0);
			System.out.println("a's original location = " + env.getLoc(a));
			while (a.collideAABB(b,env.getLoc(b).getI()-env.getLoc(a).getI(),env.getLoc(a).getJ()-env.getLoc(b).getJ()))
			{
				env.moveObject(a,v);
			}*/
			System.out.println("a's final location = " + env.getLoc(a));
			//System.out.println("	Position change vector v = " + v);
			double aI = a.getVelocity().getI();
			
			//perfectly elastic collision, momentum conserved, energy conserved (Palmer, 
			if (!b.bStatic())		//don't move b if it is static
			{
				double vxa = a.getVelocity().getI();	//temporary initial values
				double vxb = b.getVelocity().getI();
				double ma = a.getMass();
				double mb = b.getMass();
				double n1 = ma - (E*mb);
				double d = ma + mb;
				double n2 = (1+E)*mb;
				a.getVelocity().setI( ((n1*vxa) + (n2*vxb))/d );
				n1 = (1+E)*ma;
				n2 = mb-(E*ma);
				b.getVelocity().setI( ((n1*vxa) + (n2*vxb))/d );
				
				a.getVelocity().setJ(E*a.getVelocity().getJ());
				b.getVelocity().setJ(E*a.getVelocity().getJ());
			}
			else		//static object collision special case
			{
				if (v.getI() > 0)
					s = -1;
				else
					s = 1;
				System.out.println("Old velocity = " + a.getVelocity());
				a.getVelocity().setI(-E*a.getVelocity().getI());
				
				a.getVelocity().setJ(E*a.getVelocity().getJ());
				
				System.out.println("New velocity = " + a.getVelocity());
			}
			
			return true;
		}
		//if from top/bottom
		if (v.getJ() != 0 && v.getI() == 0)
		{				
			//System.out.println("Top/bottom collision...");
			
			v = env.getLoc(b).subtract(env.getLoc(a));
			if (v.getJ() > 0)
				v = v.subtract(b.getHeightVector()).subtract(a.getHeightVector());
			else
				v = v.add(b.getHeightVector()).add(a.getHeightVector());
			
			v.setI(0);
			
			//make sure it will be moved at least 1 unit
			if (v.getJ() < COLL_CORRECTION && v.getJ() > 0)
				v.setJ(COLL_CORRECTION);
			else if (v.getJ() > -COLL_CORRECTION && v.getJ() < 0)
				v.setJ(-COLL_CORRECTION);
			else if (v.getJ() == 0)
				if (env.getLoc(a).getJ() < env.getLoc(b).getJ())
					v.setJ(-COLL_CORRECTION);
				else
					v.setJ(COLL_CORRECTION);
			
			env.moveObject(a,v.scalarMult(1));
			if (!b.bStatic())
				env.moveObject(b,v.scalarMult(-1));
			//System.out.println("	Y Position change vector v = " + v);
			double aJ = a.getVelocity().getJ();
			
			if (!b.bStatic())
			{
				double vya = a.getVelocity().getJ();	//temporary vya initial value
				a.getVelocity().setJ( ((a.getVelocity().getJ()*(a.getMass()-(E*b.getMass())))/(a.getMass()+b.getMass())) +
									((b.getVelocity().getJ()*(b.getMass()*(1+E)))/(a.getMass()+b.getMass())) );
				//b.getVelocity().setJ(a.getVelocity().getJ() + aJ - b.getVelocity().getJ());
				b.getVelocity().setJ( ((b.getVelocity().getJ()*(b.getMass()-(E*a.getMass())))/(a.getMass()+b.getMass())) +
					((vya*(a.getMass()*(1+E)))/(a.getMass()+b.getMass())) );
				a.getVelocity().setI(E*a.getVelocity().getI());
				b.getVelocity().setI(E*a.getVelocity().getI());
			} else
			{
				int s;
				if (v.getJ() < 0)
					s = -1;
				else
					s = 1;
				//System.out.println("Old velocity = " + a.getVelocity());
				a.getVelocity().setJ(s*E*Math.abs(a.getVelocity().getJ()));
				
				a.getVelocity().setI(E*a.getVelocity().getI());
				
				//System.out.println("New velocity = " + a.getVelocity());
			}
			
			return true;
		}
		
		System.out.println("AABB/AABB collision error.");
		return false;
	}
	
	private Vector2D AAFromSide(AABB a, AABB b)		//returns vector pointing at direction of edge of collision
	{
		//calculate local coordinates of a (b at origin) and get object dimensions
		double xa = env.getLoc(a).getI()-env.getLoc(b).getI(),	//getpositions 
			ya = env.getLoc(a).getJ()-env.getLoc(b).getJ(), 	//in coordinates local to b
			wa = a.getWidth(), wb = b.getWidth(), ha = a.getHeight(), hb = b.getHeight(),
			//prevxa = xa-((a.getVelocity().getI()+(a.getAcceleration().getI()*dt))*dt), 
			//prevya = ya-((a.getVelocity().getJ()+(a.getAcceleration().getJ()*dt))*dt);	//old positions
			prevxa = a.oldLoc().getI()-env.getLoc(b).getI(),
			prevya = a.oldLoc().getJ()-env.getLoc(b).getJ();
		boolean bX = false, bY = false;
		//test for x-axis overlap
		/*for (int i = 0; i < BACKSTEPS; i++)
		{
			bY = AASideOverlap(-wb,wb,prevxa-wa,prevxa+wa);
			bX = AASideOverlap(-hb,hb,prevya-ha,prevya+ha);
			if (bY && !bX)
			{
				System.out.println("AAFromSide::580: vertical collision.");
				return new Vector2D(0,prevya);
			} else if (bX && !bY)
			{
				System.out.println("AAFromSide::580: horizontal collision.");
				return new Vector2D(prevxa,0);
			}
			prevxa -= (a.getVelocity().getI()+(a.getAcceleration().getI()*dt))*dt;
			prevya -= (a.getVelocity().getJ()+(a.getAcceleration().getJ()*dt))*dt;
		}*/
		
		if (AASideOverlap(xa,xa,-wb,wb))
		{
			return new Vector2D(0,ya);
		} else if (AASideOverlap(ya,ya,-hb,hb))
		{
			return new Vector2D(xa,0);
		}
		
		System.out.println("AABB came from a corner.");
		
		//velocity components will never be zero(?)
		if (prevya/a.getVelocity().getJ() >= prevxa/a.getVelocity().getI())
			return new Vector2D (prevxa,0);
		else
			return new Vector2D(0,prevya);
	}
	
	private boolean AASideOverlap(double a1, double a2, double b1, double b2)
	{	//check if two one-dimensional line segments overlap/intersect. pass in left to right
		//System.out.println(new Vector2D(a1,a2) + " " + new Vector2D(b1,b2) + ": overlap testing...");
		if (a1 < b1 && a2 < b1)		//easier to test to see if not overlapping
		{
			return false;
		}
		if (a1 > b2 && a2 > b2)
		{
			return false;
		}
		//System.out.println("Side overlap true between: " + a1 + "," + a2 + " & " +b1+","+b2);
		return true;
	}
	
	public boolean collisionAO(PointMass a, PointMass b)
	{
		return false;
	}
	
	public boolean collisionOO(PointMass a, PointMass b)
	{
		return false;
	}
	
	public static double pointLineSqDist(double ax, double ay, double bx, double by)
	{	//finds squared distance between a point and a line (local coordinates, origin is point)
		System.out.println("ax = " + ax + "; ay = " + ay + "; bx = " + bx + "; by = " + by);
		double dy = bx + ax;
		double dx = by + ay;
		double z = (ax*dx + ay*dy);
		z = z / (dx*dx + dy*dy);
		System.out.println("z = " + z);
		if (z < 0)
			z = 0;
		else if (z > 1)
			z = 1;
		System.out.println("pointLineSqDist actual dist: " + Math.sqrt((-ax + z*dx)*(-ax + z*dx) + (-ay + z*dy)*(-ay + z*dy)));
		return (-ax + z*dx)*(-ax + z*dx) + (-ay + z*dy)*(-ay + z*dy);
	}
	
	public static double pointLineSqDist(Vector2D A, Vector2D B, Vector2D C)
	{	//finds squared distance between a point and a line (location vectors)
		Vector2D AB = B.subtract(A);
		double x = C.subtract(A).dot(AB)/AB.dot(AB);
		if (x < 0)
			x = 0;
		if (x > 1)
			x = 1;
		Vector2D d = A.add(AB.scalarMult(x)).subtract(C);
		return d.getI()*d.getI() + d.getJ()*d.getJ();
	}
	
	public double getDT()
	{
		return dt;
	}
	
	public PhysicsEnv getEnv()
	{
		return env;
	}
}