/*#############################
 *	Author: Mark Walker
 *	Class: PhysicsEnv.java
 *		This is the environment, which keeps track of all objects' positions for the physics engine
 *############################*/
import java.util.*;
import java.awt.*;

public class PhysicsEnv
{
	
	private ArrayList objs,cons,dcons;		//list of objects in environment
	private final int WIDTH = 800, HEIGHT = 800, CELL_SIZE = 200;			//dimensional parameters
	
	public PhysicsEnv()
	{
		objs = new ArrayList();
		cons = new ArrayList();
		dcons = new ArrayList();
	}
	
	public PointMass getObject(Vector2D loc)		//get object according to location
	{
		for (int i = 0; i < objs.size(); i++)
			if (loc.equals(((PointMass)objs.get(i)).getLoc()))
				return (PointMass)objs.get(i);
		return null;
	}
	
	public ArrayList getAllObjects()			//return ArrayList of all objects
	{
		return objs;
	}
	
	public ArrayList getAllConstraints()
	{
		return cons;
	}
	
	public ArrayList getAllDoubleConstraints()
	{
		return dcons;
	}
	
	public void removeObject(PointMass p)
	{
		objs.remove(p);
	}
	
	public void removeObject(Constraint c)
	{
		cons.remove(c);
	}
	
	public void removeObject(DoubleConstraint c)
	{
		dcons.remove(c);
	}
	
	public void addObject(PointMass p)			//add object to environment
	{
		if (p == null)
			return;
		objs.add(p);
		System.out.println("Object added at : " + p.getLoc());
	}
	
	public void addObject(Constraint c)
	{
		if (c == null)
			return;
		cons.add(c);
		System.out.println("Constraint added at : " + c.getLoc());
	}
	
	public void addObject(DoubleConstraint c)
	{
		if (c == null)
			return;
		dcons.add(c);
	}
	
	public Vector2D getLoc(PointMass p)			//get location of a given PointMass
	{
		if (p != null)
			return p.getLoc();
		return null;
	}
	
	public Vector2D getLoc(Constraint c)			//get location of a given Constraint
	{
		if (c != null)
			return c.getLoc();
		return null;
	}
	
	public void setLoc(PointMass p, Vector2D l)
	{
		if (p == null || l== null)
			return;
		p.setLoc(l);
	}
	
	public Vector2D moveObject(PointMass p, Vector2D v)		//pass in point mass and CHANGE in position v
	{
		if (p == null || v == null)
			return v;
		Vector2D w = p.getLoc().add(v);
		
		//comment this section out to prevent objects wrapping at boundaries
		/*if (w.getI() < 0)
			w.setI((double)WIDTH);
		else if (w.getI() > (double)WIDTH)
			w.setI(0.0);
		if (w.getJ() < 0)
			w.setJ((double)WIDTH);
		else if (w.getJ() > (double)WIDTH)
			w.setJ(0);*/
		
		p.setLoc(w);
		return w;
	}
	
	public Vector2D nextLoc(PointMass p,double dt)
	{
		//System.out.println("nextLoc:: finding next location; current = " + getLoc(p) + "; dS = " + p.fakeAct(dt));
		return getLoc(p).add(p.fakeAct(dt));
	}
}