/*#############################
 *	Author: Mark Walker
 *	Class: PointMass.java
 *		This is the parent class of all bounding areas. It keeps track of its own acceleration and velocity.
 *############################*/

//0 = pointmass
//1 = circle
//2 = AABB
//3 = OBB
//4 = RightTriangle

public class PointMass
{
	private Vector2D v,a;
	protected double m, omega, I;
	private int CLASS = 0;
	private boolean bFlag, bCollide, bStatic, bSleeping;
	private Vector2D loc, oldLoc;
	
	public PointMass(double mass, Vector2D l)
	{
		v = new Vector2D();
		a = new Vector2D();
		m = mass;
		bSleeping = false;
		omega = 0;
		loc = l;
		oldLoc = loc;
		bFlag = false;
		bCollide = true;
		bStatic = false;
		I = .5*m;
	}
	
	public boolean bSleeping()
	{
		return bSleeping;
	}
	
	public void sleep()
	{
		stop();
		bSleeping = true;
	}
	
	public void stop()
	{
		v.setI(0);
		v.setJ(0);
	}
	
	public void wake()
	{
		bSleeping = false;
	}
	
	public double getKE()
	{
		return .5*m*v.squaredMag();
	}
	
	public double actualKE(double dt)
	{
		return .5*m*v.add(a.scalarMult(dt)).squaredMag();
	}
	
	public Vector2D oldLoc()
	{
		return oldLoc;
	}
	
	public double getOmega()
	{
		return omega;
	}
	
	public double getI()
	{
		return I;
	}
	
	public void setOmega(double d)
	{
		omega = d;
	}
	
	public void addOmega(double d)
	{
		omega += d;
	}
	
	public void setLoc(Vector2D l)		//set location to vector l
	{
		oldLoc = loc;
		loc = l;
	}
	
	public void setMass(double d)
	{
		m = d;
	}
	
	public void collisionOff()
	{
		bCollide = false;
	}
	
	public void collisionOn()
	{
		bCollide = true;
	}
	
	public boolean bCollidable()
	{
		return bCollide;
	}
	
	public boolean bStationary()
	{
		return bStatic;
	}
	
	public Vector2D getLoc()
	{
		return loc;
	}
	
	public void reverseX()
	{
		v.setI(-v.getI());
	}
	
	public void reverseY()
	{
		v.setJ(-v.getJ());
	}
	
	public boolean collideCircle(BoundingCircle b, double x, double y)
	{
		if (x*x + y*y < b.getRadius()*b.getRadius())
			return true;
		return false;
	}
	
	public boolean collideOBB(OBB b, double x, double y)
	{
		return false;
	}
	
	public boolean collideAABB(AABB b, double x, double y)
	{
		return false;
	}
	
	public Vector2D act(double dt)
	{
		if (bStatic)
			return new Vector2D(0,0);
		Vector2D b = a.scalarMult(dt);
		v = v.add(b);
		v.round();
		//System.out.println("PointMass::40: acting with velocity " + v + "; dt = " + dt);
		return v.scalarMult(dt);
	}
	
	public Vector2D fakeAct(double dt)
	{	//provides what will be the next position without actually changing anything
		if (bStatic)
			return new Vector2D(0,0);
		Vector2D b = a.scalarMult(dt);
		b = v.add(b);
		return b.scalarMult(dt);
	}
	
	public void applyForce(Vector2D n)
	{
		n.scalarDiv(m);
		v = v.add(n);
	}
	
	public void accelerate(Vector2D d)
	{
		a = a.add(d);
	}
	
	public void switchDir()
	{
		v = v.scalarMult(-1.0);
	}
	
	public void deccelerate(Vector2D d)
	{
		a = a.subtract(d);
	}
	
	public void setVelocity(Vector2D vel)
	{
		v = vel;
		//System.out.println("PointMass velocity set to: " + v);
	}
	
	public void accelerate(double mag, double theta)
	{
		double x = mag*Math.cos(theta);
		double y = mag*Math.sin(theta);
		a = a.add(new Vector2D(x,y));
	}
	
	public double getMass()
	{
		return m;
	}
	
	public Vector2D getVelocity()
	{
		return v;
	}
	
	public Vector2D getAcceleration()
	{
		return a;
	}
	
	public void setSpeed(double d)
	{
		v.setMag(d);
	}
	
	public void setAccel(double d)
	{
		a.setMag(d);
	}
	
	public void setAccel(double x, double y)
	{
		a.set(x,y);
	}
	
	public void setAccel(Vector2D acc)
	{
		a = acc;
	}
	
	public boolean getFlag()
	{
		return bFlag;
	}
	
	public void unflag()
	{
		bFlag = false;
	}
	
	public void flag()
	{
		bFlag = true;
	}
	
	public void setStatic(boolean b)
	{
		bStatic = b;
	}
	
	public boolean bStatic()
	{
		return (bStatic || bSleeping);
	}
	
	public int CLASS()
	{
		return 0;
	}
}