/*#############################
 *	Author: Mark Walker
 *	Class: RigidRope.java
 *		Class for rigid, massless rope constraints
 *############################*/

import java.util.*;

public class RigidRope implements Constraint
{
	protected double l;
	protected PointMass obj;
	protected Vector2D loc;
	protected final double DAMP = 1;
	
	public RigidRope(PointMass o, Vector2D coords, double length)
	{
		l = length;
		obj = o;
		loc = coords;
	}
	
	public boolean updateConstraint(double dt, PhysicsEnv e)
	{
		Vector2D objLoc = e.getLoc(obj).subtract(loc);
		if (objLoc.squaredMag() <= l*l)
		{
			return false;
		}
		
		/*Vector2D r1 = objLoc.scalarDiv(objLoc.magnitude());
		Vector2D v = new Vector2D(obj.getVelocity().getJ(),obj.getVelocity().getI()).scalarMult(obj.getVelocity().cross(r1));
		v = v.scalarDiv(obj.getVelocity().magnitude());
		if (v.dot(obj.getVelocity()) < 0)
			v = v.scalarMult(-1);
		//System.out.println("New velocity " + v);
		obj.getVelocity().setI(v.getJ());
		obj.getVelocity().setJ(v.getI());
		
		Vector2D f = objLoc.scalarDiv(-objLoc.magnitude());
		f.setMag(DAMP*obj.getMass()*obj.getVelocity().squaredMag()/(l*50));
		//System.out.println("Original force " + f);
		if ((dt*dt*f.magnitude())/obj.getMass() > l)
		{
			f.setMag(0);
			obj.getVelocity().setI(0);
			obj.getVelocity().setJ(0);
		}
		
		obj.applyForce(f);*/
		
		Vector2D n = objLoc.scalarDiv(objLoc.magnitude());
		//System.out.println("Normal vector N = " + n);
		//System.out.println("	v * n = " + obj.getVelocity().dot(n) + "; v = " + obj.getVelocity());
		Vector2D dv = n.scalarMult(obj.getVelocity().dot(n));
		//System.out.println("	Response velocity dv = " + dv);
		dv = obj.getVelocity().subtract(dv);
		
		obj.getVelocity().setI(dv.getI());
		obj.getVelocity().setJ(dv.getJ());
		
		//System.out.println("	Force applied " + f);
		/*Vector2D r = objLoc.scalarDiv(-objLoc.magnitude());
		int n = 0;
		while (objLoc.squaredMag() > l*l && n < 100)
		{
			//System.out.println("Object moved back by " + r);
			e.moveObject(obj,r);
			objLoc = objLoc.add(r);
			n++;
		}*/
		
		if (objLoc.squaredMag() > l*l)
		{
			Vector2D r = objLoc.scalarDiv(-objLoc.magnitude());
			r.setMag(objLoc.magnitude()-l);
			//System.out.println("	Object moved back by " + r);
			e.moveObject(obj,r);
		}
		
		return true;
	}
	
	public Vector2D getLoc()
	{
		return loc;
	}
	
	public void addLength(double d)
	{
		l += d;
		if (l < 0)
			l = 0;
	//	System.out.println("l = " + l);
	}
	
	public double length()
	{
		return l;
	}
	
	public PointMass getObject()
	{
		return obj;
	}
	
	public void setLoc(Vector2D v)
	{
		loc = v;
	}
	
	public double getK()
	{
		return 0;
	}
	
	public void setK(double j)
	{
	}
}