import java.util.*;

/*#############################
 *	Author: Mark Walker
 *	Class: RopeGame.java
 *		A simple game that lets you shoot a ninja rope to move around
 *############################*/

public class RopeGame implements GameEngine
{
	private PhysicsEngine2 phys;
	
	private int HEIGHT = 50, WIDTH = 50, MASS = 10, MBOX = 1000, GRAVITY = 4000, V_EXPL = 2000, FORCE = 100, RADIUS = 20,
		ROWS = 5, COLS = 5, SPACE = 30, OFFSET = 200, AIM_LENGTH = RADIUS;
	private final double K = 1, B = .2, BX = .01, BY = .01, X0 = 0, LENGTH = 100.0, AIM_SPEED = .1,ROPE_SPEED = 6;
	
	BoundingCircle player;
	Constraint rope;
	Vector2D aim;
	private double rope_length;
	private boolean bStatic, bBoxes;
	
	public RopeGame()
	{
		phys = new PhysicsEngine2();
		
		bStatic = false;
		bBoxes = true;
		
		rope_length = LENGTH;
		
		aim = new Vector2D(AIM_LENGTH,0);
		
		AABB b = new AABB(1,900,200,new Vector2D(400,800));		//border boxes
		b.setStatic(true);
		AABB f = new AABB(1,900,200,new Vector2D(400,0));
		f.setStatic(true);
		AABB d = new AABB(1,200,900,new Vector2D(0,400));
		d.setStatic(true);
		AABB e = new AABB(1,200,900,new Vector2D(800,400));
		e.setStatic(true);
		getEnv().addObject(b);
		getEnv().addObject(f);
		getEnv().addObject(d);
		getEnv().addObject(e);
		
		BoundingCircle g = new BoundingCircle(1,100,new Vector2D(300,300));
		g.setStatic(true);
		//getEnv().addObject(g);
		
		player = new BoundingCircle(MBOX,RADIUS,new Vector2D(600,600));
		getEnv().addObject(player);
		player.accelerate(new Vector2D(0,GRAVITY));
		
		rope = null;
		
	}
	
	public void shootRope()
	{
		getEnv().removeObject(rope);
		//System.out.println("Shooting rope: player loc: " + getEnv().getLoc(player) + "; tracing from loc: " + getEnv().getLoc(player).add(aim.scalarMult(2)));
		Vector2D hit = PhysicsEngine2.rayTrace(getEnv().getLoc(player).add(aim.scalarMult(1.1)),aim,getEnv());
		if (hit == null)
			return;
		hit = hit.add(aim.scalarMult(1.1));
		//System.out.println("Attaching rope at: " + hit.add(getEnv().getLoc(player)));
		rope_length = hit.magnitude();
		//rope = new BendySpring(player,new Vector2D(400,150),K,X0,rope_length,BX,BY);
		//rope = new RotationalSpring(player,new Vector2D(400,110),K,rope_length,B);
		rope = new RigidRope(player,hit.add(getEnv().getLoc(player)),rope_length);
		getEnv().addObject(rope);
	}
	
	public void releaseRope()
	{
		getEnv().removeObject(rope);
		rope = null;
	}
	
	public void ropeAction()
	{
		if (rope == null)
			shootRope();
		else
			releaseRope();
	}
	
	public void changeStaticMode()
	{
		System.out.println("Static mode changed.");
		bStatic = !bStatic;
	}
	
	public void contractRope()
	{
		if (rope != null && rope.length() > RADIUS)
		{
			rope.addLength(-ROPE_SPEED);
			//moveAlongRope(-ROPE_SPEED);
		}
	}
	
	public void addRope()
	{
		if (rope != null)
		{
			rope.addLength(ROPE_SPEED);
			//moveAlongRope(ROPE_SPEED);
		}
	}
	
	private void moveAlongRope(double d)
	{
		Vector2D w = getEnv().getLoc(rope).subtract(getEnv().getLoc(player));
		w = w.scalarMult(-d/w.magnitude());
		getEnv().moveObject(player,w);
	}
	
	public boolean bBoxes()
	{
		return bBoxes;
	}
	
	public boolean bRoped()
	{
		return (rope != null);
	}
	
	public void addObject(int x, int y)
	{
		if (bBoxes)
			addBox(x,y);
		else
			addCircle(x,y);
	}
	
	public void toggleBoxes()
	{
		//System.out.println("Object type toggled.");
		//bBoxes = !bBoxes;
	}
	
	public void addBox(int x, int y)
	{
		//System.out.println("Adding box at location (" + x + ", " + y + ")");
		AABB box = new AABB(MASS,WIDTH,HEIGHT,new Vector2D(x,y));
		//getEnv().addObject(box);
		box.setAccel(new Vector2D(0,GRAVITY));	
		if (bStatic)
			box.setStatic(true);	
	}
	
	public void addCircle(int x, int y)
	{
		//System.out.println("Adding circle at location (" + x + ", " + y + ")");
		BoundingCircle circle = new BoundingCircle(MASS,RADIUS,new Vector2D(x,y));
		getEnv().addObject(circle);
		circle.setAccel(new Vector2D(0,GRAVITY));	
		if (bStatic)
			circle.setStatic(true);	
	}
	
	public BoundingCircle getPlayer()
	{
		return player;
	}
	
	public Vector2D getAim()
	{
		return aim;
	}
	
	public void aimCW()
	{
		Vector2D a = aim.rotateCCW(-AIM_SPEED);
		//System.out.println("Rotating aim CW from: " + aim + " to " + a);
		aim.set(a.getI(),-a.getJ());
	}
	
	public void aimCCW()
	{
		Vector2D a = aim.rotateCCW(AIM_SPEED);
		aim.set(a.getI(),-a.getJ());
	}
	
	/*public void explosion(int x, int y)
	{
		//System.out.println("EXPLOSION!");
		ArrayList l = getEnv().getAllObjects();
		AABB box;
		BoundingCircle circle;
		for (int i = 0; i < l.size(); i++)
		{
			if (((PointMass)l.get(i)).CLASS() == 2)
			{
				box = (AABB)l.get(i);
				if (!box.bStatic() || box.bSleeping())
				{
					box.setVelocity(new Vector2D(V_EXPL*-Math.sin(getEnv().getLoc(box).getI()/2),
						V_EXPL*-Math.cos(getEnv().getLoc(box).getJ()/2)));
					box.wake();
				}
			} else if (((PointMass)l.get(i)).CLASS() == 1)
			{
				circle = (BoundingCircle)l.get(i);
				if (!circle.bStatic() || circle.bSleeping())
				{
					circle.setVelocity(new Vector2D(V_EXPL*-Math.sin(getEnv().getLoc(circle).getI()/2),
					//-V_EXPL));
					V_EXPL*-Math.cos(getEnv().getLoc(circle).getJ()/2)));
					circle.wake();
				}
			}
		}
	}*/
	
	public void hitBox(Vector2D d)
	{
		if (rope != null)
			player.applyForce(d.scalarMult(FORCE));
	}
	
	public void act()
	{
		phys.act();
		if (player.bSleeping())
			player.wake();
		ArrayList l = getEnv().getAllObjects();
		double y = getEnv().getLoc(player).getJ();
		PointMass p;
		for (int i = 0; i < l.size() && player.getFlag(); i++)
		{
			p = (PointMass)l.get(i);
			if (y > getEnv().getLoc(p).getJ())
				p.wake();
		}
	}
	
	public PhysicsEnv getEnv()
	{
		return phys.getEnv();
	}
}