/*#############################
 *	Author: Mark Walker
 *	Class: RotationalSpring.java
 *		Class for springs that will rotate about a pivot point and oscillate rigidly
 *############################*/

import java.util.*;

public class RotationalSpring extends RigidRope implements Constraint
{
	protected double k,b;			//b is the damping coefficient; k is spring coefficient
	
	public RotationalSpring(PointMass o, Vector2D coords, double k1,double length, double damp)
	{
		super(o,coords,length);
		k=k1;
		b = damp;
	}
	
	public boolean updateConstraint(double dt, PhysicsEnv e)
	{
		//System.out.println("e = " + e + "; e.getLoc(obj) = " + e.getLoc(obj));
		Vector2D r = e.getLoc(obj).subtract(loc);
		
		//double fx = k*(-r.getI()+x0);
		//double fy = k*(-r.getJ()+y0);
		double mag = r.magnitude();
		r = r.scalarDiv(mag);
		Vector2D f = r.scalarMult(-k*(-l+mag)).subtract(r.scalarMult(b*obj.getVelocity().magnitude()));
		
		obj.applyForce(f);
		
		//System.out.println("Applying spring force: " + new Vector2D(fx,fy) + "; k = " + k);
		
		return true;
	}
	
	public double getK()
	{
		return k;
	}
	
	public void setK(double j)
	{
		k = j;
	}
}