/*#############################
 *	Author: Mark Walker
 *	Class: TheDemonstration.java
 *		All-in-one demonstration applet
 *############################*/
import java.util.*;
public class TheDemonstration implements GameEngine
{
	private PhysicsEngine2 phys;
	private Vector2D anchor;
	
	private final int V_EXPL = 500, FORCE = 100;
	private final double K_INCR = 0.1, B_INCR = 0.01;
	private int h = 20, w = 20,g = 3000, cType = 0;
	private double  m = 1, r = 20, k = .5, B = .1, r0 = 10;
	public boolean bStatic, bBoxes, buildMode;
	private PointMass anchorObj;
	
	//cType key: 0 = RigidRope, 1 = RotationalSpring, 2 = BendySpring, 3 = DoubleRope,
	//			 4 = DoubleSpring, 5 = DoubleBendy
	
	public TheDemonstration()
	{
		phys = new PhysicsEngine2();
		anchor = null;
		anchorObj = null;
		bStatic = false;
		bBoxes = false;
		buildMode = true;
		
		AABB b = new AABB(1,900,200,new Vector2D(400,800));		//border boxes
		b.setStatic(true);
		AABB f = new AABB(1,900,200,new Vector2D(400,0));
		f.setStatic(true);
		AABB d = new AABB(1,200,900,new Vector2D(0,400));
		d.setStatic(true);
		AABB e = new AABB(1,200,900,new Vector2D(800,400));
		e.setStatic(true);
		getEnv().addObject(b);
		getEnv().addObject(f);
		getEnv().addObject(d);
		getEnv().addObject(e);
	}
	
	public void toggleBuildMode()
	{
		buildMode = !buildMode;
	}
	
	public void changeStaticMode()
	{
		System.out.println("Static mode changed.");
		bStatic = !bStatic;
	}
	
	public boolean bBoxes()
	{
		return bBoxes;
	}
	
	public double getK()
	{
		return k;
	}
	
	public void addK(int s)
	{
		k += s*K_INCR;
		if (k <= 0)
			k = 0;
		System.out.println("k = " + k);
		Constraint c;
		for (int i = 0; i < getEnv().getAllConstraints().size(); i++)
		{
			c = (Constraint)getEnv().getAllConstraints().get(i);
			c.setK(k);
		}
		
		DoubleConstraint d;
		for (int i = 0; i < getEnv().getAllDoubleConstraints().size(); i++)
		{
			d = (DoubleConstraint)getEnv().getAllDoubleConstraints().get(i);
			d.setK(k);
		}
	}
	
	public void addObject(int x, int y)
	{
		if (anchor != null)
		{
			anchor = null;
			return;
		}
		if (bBoxes)
			addBox(x,y);
		else
			addCircle(x,y);
	}
	
	public void toggleBoxes()
	{
		//System.out.println("Object type toggled.");
		bBoxes = !bBoxes;
	}
	
	public void addBox(int x, int y)
	{
		//System.out.println("Adding box at location (" + x + ", " + y + ")");
		AABB box = new AABB(m,w,h,new Vector2D(x,y));
		getEnv().addObject(box);
		box.setAccel(new Vector2D(0,g));	
		if (bStatic)
			box.setStatic(true);	
	}
	
	public void addCircle(int x, int y)
	{
		//System.out.println("Adding circle at location (" + x + ", " + y + ")");
		BoundingCircle circle = new BoundingCircle(m,r,new Vector2D(x,y));
		getEnv().addObject(circle);
		circle.setAccel(new Vector2D(0,g));	
		if (bStatic)
			circle.setStatic(true);	
	}
	
	//cType key: 0 = RigidRope, 1 = RotationalSpring, 2 = BendySpring, 3 = DoubleRope,
	//			 4 = DoubleSpring, 5 = DoubleBendy
	
	public String getConstraintType()
	{
		if (cType == 0)
			return "Rope";
		if (cType == 1)
			return "Rotational Spring";
		if (cType == 2)
			return "Bendy Spring";
		if (cType == 3)
			return "Double Rope";
		if (cType == 4)
			return "Double Spring";
		if (cType == 5)
			return "Double Bendy";
		return null;
	}
	
	public void changeConstraint()
	{
		cType++;
		if (cType > 5)
			cType = 0;
	}
	
	public void rightClick(int x, int y)
	{
		Vector2D click = new Vector2D(x,y);
		ArrayList l = getEnv().getAllObjects();
		AABB a;
		BoundingCircle c;
		PointMass p;
		Vector2D pLoc;
		
		for (int i = 0; i < l.size(); i++)
		{
			p = (PointMass)l.get(i);
			pLoc = click.subtract(getEnv().getLoc(p));
			if (p.CLASS() == 1)
			{
				c = (BoundingCircle)p;
				//System.out.println("Testing circle; pLoc = " + pLoc);
				if (pLoc.squaredMag() <= c.getRadius()*c.getRadius())
				{
					if (anchor == null && anchorObj == null)
					{
						if (cType >= 3 && cType <= 5)
						{
							anchorObj = p;
							System.out.println("assigned anchorObj = " + anchorObj);
							return;
						}
								
					} else
					{
						System.out.println("anchorObj = " + anchorObj);
						if (anchorObj != null)
							addDoubleConstraint(anchorObj,p);
						else
							addConstraint(p);
						
						return;
					}
				}
			} else if (p.CLASS() == 2)
			{
				a = (AABB)p;
				//System.out.println("Testing AABB; pLoc = " + pLoc);
				if (PhysicsEngine2.AASideOverlap(pLoc.getI(),pLoc.getI(),-a.getWidth(),a.getWidth())
					&& PhysicsEngine2.AASideOverlap(pLoc.getJ(),pLoc.getJ(),-a.getHeight(),a.getHeight()))
				{
					if (anchor == null && anchorObj == null)
					{
						if (cType >= 3 && cType <= 5)
						{
							anchorObj = p;
							return;
						}
								
					} else
					{
						if (anchorObj != null)
							addDoubleConstraint(anchorObj,p);
						else
							addConstraint(p);
						return;
					}
				}
			}
		}
		if (anchor == null && anchorObj == null && (cType >= 0 && cType <= 2))
			anchor = click;
		else if (anchor == null)
			anchorObj = null;
	}
	
	public Vector2D getAnchor()
	{
		if (anchorObj != null)
			return getEnv().getLoc(anchorObj);
		else
			return anchor;
	}
	
	public void addConstraint(PointMass p)
	{
		if (p == null)
			return;
		double d = getEnv().getLoc(p).subtract(anchor).magnitude();
		Constraint c = null;
		if (cType == 0)
		{
			c = new RigidRope(p,anchor,d);
		} else if (cType == 1)
		{
			c = new RotationalSpring(p,anchor,d,k,B);
		} else if (cType == 2)
		{
			Vector2D loc = getEnv().getLoc(p).subtract(anchor);
			c = new BendySpring(p,anchor,k,loc.getI(),loc.getJ(),B,B);
		}
		if (c != null)
			getEnv().addObject(c);
		
		anchor = null;
	}
	
	public void addDoubleConstraint(PointMass a, PointMass b)
	{
		if (a == null || b == null)
			return;
		DoubleConstraint c = null;
		if (cType == 3)		//double rope
		{
			c = new DoubleConstraint(a,b,getEnv().getLoc(a).subtract(getEnv().getLoc(b)).magnitude());
		}
		else if (cType == 4)		//double spring
		{
			c = new DoubleSpring(a,b,k,
					getEnv().getLoc(a).subtract(getEnv().getLoc(b)).magnitude()/2,B);
		}
		else if (cType == 5)		//double bendy
		{
			c = new DoubleBendy(a,b,
					getEnv().getLoc(b).subtract(getEnv().getLoc(a)),k,B);
		}
		if (c != null)
			getEnv().addObject(c);
		
		anchorObj = null;
	}
	
	public void removeObject(int x, int y)
	{
		Vector2D click = new Vector2D(x,y);
		ArrayList l = getEnv().getAllObjects();
		AABB a;
		BoundingCircle c;
		PointMass p;
		Vector2D pLoc;
		
		for (int i = 0; i < l.size(); i++)
		{
			p = (PointMass)l.get(i);
			pLoc = click.subtract(getEnv().getLoc(p));
			if (p.CLASS() == 1)
			{
				c = (BoundingCircle)p;
				//System.out.println("Testing circle; pLoc = " + pLoc);
				if (pLoc.squaredMag() <= c.getRadius()*c.getRadius())
				{
					getEnv().removeObject(p);
				}
			} else if (p.CLASS() == 2)
			{
				a = (AABB)p;
				//System.out.println("Testing AABB; pLoc = " + pLoc);
				if (PhysicsEngine2.AASideOverlap(pLoc.getI(),pLoc.getI(),-a.getWidth(),a.getWidth())
					&& PhysicsEngine2.AASideOverlap(pLoc.getJ(),pLoc.getJ(),-a.getHeight(),a.getHeight()))
				{
					getEnv().removeObject(p);
				}
			}
		}	
	}
	
	public void act()
	{
		if (!buildMode)
			phys.act();
	}
	
	public PhysicsEnv getEnv()
	{
		return phys.getEnv();
	}
	
	public void setHeight(double h)
	{
		
	}
}