import java.util.*;

public class ThreeBodyProblem implements GameEngine
{
	private PhysicsEngine2 phys;
	private PointMass[] objs;
	
	//############## MAIN PARAMETERS ################
	private final int NUM = 100, MASS = 1000, BIG_MASS = 1000000, RADIUS = 200;
	private final double SPEED = 500;
	
	public ThreeBodyProblem()
	{
		phys = new PhysicsEngine2();
		objs = new PointMass[NUM+1];
		double t = 0;
		for (int i = 0; i < objs.length-1; i++)
		{
			t += 6.28318/(objs.length-1);
			//objs[i] = new BoundingCircle(MASS,7,new Vector2D(Math.cos(t)*RADIUS + 400, Math.sin(t)*RADIUS + 400));
			objs[i] = new AABB(MASS,4,4,new Vector2D(Math.cos(t)*RADIUS + 400, Math.sin(t)*RADIUS + 400));
			getEnv().addObject(objs[i]);
			//objs[i].collisionOff();		//comment out to try collision detection
			objs[i].setVelocity(new Vector2D(-Math.sin(t)*SPEED,Math.cos(t)*SPEED));
		}
		objs[NUM] = new BoundingCircle(BIG_MASS,25,new Vector2D(400,400));
		//objs[NUM] = new AABB(BIG_MASS,5,5,new Vector2D(400,400));
		getEnv().addObject(objs[NUM]);
		//objs[NUM].collisionOff();			//collision off for middle
		objs[NUM].setSpeed(1);
	}
	
	public void act()
	{
		double r;
		Vector2D p = new Vector2D(0,0);
		for (int i = 0; i < objs.length; i++)
		{
			objs[i].setAccel(0,0);		//reset acceleration vector every time to 0
			for (int j = 0; j < objs.length; j++)
			{
				if (j != i)		//add accelerations in the direction of each other object, (M/R^2)
				{
					p.set(objs[j].getLoc().getI()-objs[i].getLoc().getI(),objs[j].getLoc().getJ()-objs[i].getLoc().getJ());
					r = p.magnitude()/10;
					//System.out.println("magnitude = " + r);
					if (r < 1)
						r = 1;
					p.setMag((objs[j].getMass())/(r*r));
					//System.out.println("Vector p" + i + j + " = " + p);
					objs[i].accelerate(p);
				}
			}
		}
		phys.act();
	}
	
	public PhysicsEnv getEnv()
	{
		return phys.getEnv();
	}
}