/*#############################
 *	Author: Mark Walker
 *	Class: Vector2D.java
 *		This object is a vector that points to a point on a a rectangular cartesian coordinate grid
 *############################*/
public class Vector2D
{
	private double i,j;
	
	public Vector2D()
	{
		init(0,0);
	}
	
	public Vector2D(double nI,double nJ)
	{
		init(nI,nJ);
	}
	
	private void init(double nI, double nJ)
	{
		i = nI;
		j = nJ;
	}
	
	public double dot(Vector2D v)
	{
		return (i*v.getI()) + (j*v.getJ());
	}
	
	public Vector2D add(Vector2D v)
	{
		double aI = i + v.getI();
		double aJ = j + v.getJ();
		return new Vector2D(aI,aJ);
	}
	
	public Vector2D addI(double d)
	{
		return new Vector2D(i+d,j);
	}
	
	public Vector2D addJ(double d)
	{
		return new Vector2D(i,j+d);
	}
	
	public double squaredMag()
	{
		return i*i + j*j;
	}
	
	public void round()
	{
		int ii = (int)i;
		int ji = (int)j;
		i = (double)ii;
		j = (double)ji;
	}
	
	public Vector2D subtract(Vector2D v)
	{
		double aI = i - v.getI();
		double aJ = j - v.getJ();
		return new Vector2D(aI,aJ);
	}
	
	public Vector2D scalarMult(double d)
	{
		return new Vector2D(i*d,j*d);
	}
	
	public Vector2D scalarDiv(double d)
	{
		return new Vector2D(i/d,j/d);
	}
	
	public double magnitude()
	{
		double d = Math.sqrt(i*i + j*j);
		//if (d == 0)
			//System.out.println("WARNING: 0 magnitude vector!");		//magnitude is often in a denominator
		return d;
	}
	
	public double angle(Vector2D v)
	{
		double a = dot(v), r1 = this.magnitude(), r2 = v.magnitude();
		if (r1 == 0 || r2 == 0)
			return 0;
		a /= (r1 * r2);
		
		return Math.acos(a);
	}
	
	public double getAngle()		//returns angle CCW from +x axis between 0 and 2pi
	{
		double a = angle(new Vector2D(1,0));
		//if (a < 0)
		//	a += 6.28;				//if arcsin returns negative angle, add 2pi to it
		//System.out.println("Get angle = " + a + " radians");
		return a;
	}
	
	public Vector2D rotateCCW(double a)		//rotate vector counter-clockwise
	{
		//System.out.println("Rotating vector CCW " + (getAngle() + a) + " radians");
		return setAngle(getAngle() + a);
	}
	
	public void setMag(double d)
	{
		if (magnitude() == 0)
		{
			i = d;
			j = 0;
			return;
		}
		double r = d / magnitude();
		i *= r;
		j *= r;
	}
	
	public Vector2D setAngle(double a)		//sets angle between vector and +x axis
	{
		double m = magnitude();
		//System.out.println("Rotating vector to " + a + " radians");
		return new Vector2D(m * Math.cos(a), m * Math.sin(a));
	}
	
	public double cross(Vector2D v)		//returns z component of cross product
	{
		return i*v.getJ() - j*v.getI();
	}
	
	public void set(double a, double b)
	{
		i = a;
		j = b;
	}
	
	public double getJ()
	{
		return j;
	}
	
	public double getI()
	{
		return i;
	}
	
	public void setI(double d)
	{
		i = d;
	}
	
	public void setJ(double d)
	{
		j = d;
	}
	
	public boolean equals(Vector2D v)
	{
		if (v != null)
			return (i == v.getI() && j == v.getJ());
		return false;
	}
	
	public String toString()
	{
		return "<" + i + "," + j + ">";
	}
}